/*
 * @(#)JPassGenGUI.java  2002-07-29
 *
 * Version 1.04
 *
 * Copyright (C) 26 juillet 2002 hughes monget
 * hmonget@users.sourceforge.net
 * http://jpassgen.sourceforge.net/
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package net.sourceforge.jpassgen;

// For the look and feel.
import com.incors.plaf.kunststoff.KunststoffLookAndFeel;
import com.incors.plaf.kunststoff.themes.KunststoffNotebookTheme;

// For GUI events.
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
//import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

// For GUI components.
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Font;
import java.awt.Point;

// For icon.
import java.net.URL;

// For version number and entropy format.
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;

// For GUI components.
import javax.swing.border.Border;
import javax.swing.BorderFactory; 
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSlider;
//import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.UIManager;
import javax.swing.SwingUtilities;

// For i8n.
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import javax.swing.AbstractButton;


/**
 * This class is the main application of the password generator project:
 * It is a swing <b>gui</b> version.
 *
 * @author hughes monget
 */
public class JPassGenGUI extends JFrame {

    //////////////////
    // CLASS FIELDS //
    //////////////////

    /**
     * The icon of the window.
     */
    static final String ICON = 
            "/org/javalobby/icons/20x20/DocumentLock.gif";

    /**
     * We set the look and feel or not.
     */
    static final boolean LNF = true;

    /**
     * The version of the JPassGen.
     */
    private static final float VERSION = 1.04f; 

    //public static final int WIDTH = 360;
    //public static final int HEIGHT = WIDTH / 4 * 3;

    ///////////////////
    // CLASS METHODS //
    ///////////////////

    /**
     * The class is launched as an application. 
     */
    private static boolean isApplication = false;

    /** 
     * Main function . <br>
     *
     * @param args command line arguments. 
     */ 
    public static void main(String args[]) {

        JPassGenGUI.isApplication = true;

        // We set te look and feel.
        JPassGenGUI.setLookAndFeel();
        final String message = LanguageServer.getString("IDS_SPLASH_WAITING");

        final SplashWindow splash = new SplashWindow("JPassGen");

        // We create the splash.
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                splash.setText(message);
                splash.setVisible(true);
            }
        });

        // We create the main frame and initialize the UI.
        String title = LanguageServer.getString("IDS_TITLE_APP");
        final JPassGenGUI jpg = new JPassGenGUI(title);

        jpg.initializeUi();

        // We destroy the splash and show main frame.
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                splash.setVisible(false);
                splash.dispose();
                jpg.setVisible(true);
            }
        });

        // Lazy initialization of the Random algorithm
        // In jdk 1.3, it took a little time ...
        jpg.initializeSeed();
    }

    /**
     * We set a look and feel.
     */
    static void setLookAndFeel() {

        if (JPassGenGUI.LNF) {
            try {
                // "Kunstoff I like" 
                KunststoffLookAndFeel lnf = new KunststoffLookAndFeel();
                // "Standard Kunstoff"
                //lnf.setCurrentTheme(
                //      new com.incors.plaf.kunststoff.KunststoffTheme());
                lnf.setCurrentTheme(new KunststoffNotebookTheme());
                UIManager.setLookAndFeel(lnf);
            } catch (UnsupportedLookAndFeelException ex) {
                System.err.println("** UI:UnsupportedLookAndFeelException: "
                        + ex.getMessage());
            } catch (Exception ex) {
                System.err.println("** UI:Exception: " + ex.getMessage());
            }
        } else {
            //UIManager.getLookAndFeelDefaults().put("ClassLoader", 
            //        ClassLoader.getSystemClassLoader());
            String mac = "com.sun.java.swing.plaf.mac.MacLookAndFeel";
            String win = "com.sun.java.swing.plaf.windows.WindowsLookAndFeel";
            String mot = "com.sun.java.swing.plaf.motif.MotifLookAndFeel";
            String met = "javax.swing.plaf.metal.MetalLookAndFeel";
            try {
                // We set the look and feel of your system.
                UIManager.setLookAndFeel(
                        UIManager.getSystemLookAndFeelClassName());
                //UIManager.setLookAndFeel(met);
            } catch (ClassNotFoundException e) {
                System.err.println("** UI:ClassNotFoundException !");
            } catch (InstantiationException e) {
                System.err.println("** UI:InstantiationException !");
            } catch (IllegalAccessException e) {
                System.err.println("** UI:IllegalAccessException !");
            } catch (UnsupportedLookAndFeelException e) {
                System.err.println("** UI:UnsupportedLookAndFeelException!");
            }
        }
    }

    /////////////////////
    // INSTANCE FIELDS //
    /////////////////////

    /**
     * This is a map to link identifier strings and visual components. 
     */
    private HashMap _idsMap = null;

    /** 
     * The seed is initialized or not.
     */
    private boolean _bSeedOk = false;

    /** 
     * An array of check boxes.
     */
    private JCheckBox[] _arrCheckBoxes;

    // TO DO: A remplacer ventuellement par un BitSet
    /** 
     * An array of booleans "Checked" characters.
     */
    private boolean[] _arrChecked = {false, false, false, false};

    /** 
     * An array of check menus items.
     */
    private JCheckBoxMenuItem[] _arrCheckMenus;

    /** 
     * An array of bits for mask.
     */
    private int[] _arrMasks = {RandomPassword.LOWERS, RandomPassword.UPPERS,
            RandomPassword.DIGITS, RandomPassword.OTHERS};

    /** 
     * The "Generate" button.
     */
    private JButton _btnGenerate;

    /**
     * The "Generate" item menu.
     */
    private JMenuItem _menuGenerate;

    /**
     * The "Select One" item menu.
     */
    private JMenuItem _menuSelectOne;

    /**
     * The "Select All" item menu.
     */
    private JMenuItem _menuSelectAll;

    /** 
     * The slider for the length.
     */
    private JSlider _sliderLength;

    /** 
     * The text area where will be the generated passwords. 
     */
    private JTextArea _textPasswords;

    /** 
     * The "About" message box. 
     */
    private JOptionPane _jopAbout = null;

    /** 
     * The "Infos" message box.
     */
    private JOptionPane _jopInfos = null;

    /** 
     * The status bar.
     */
    private StatusBar _statusBar = null;

    /** 
     * The last length of generated password.
     */
    private int _lastLength = 0;


    //////////////////
    // CONSTRUCTORS //
    //////////////////

    /**
     * We call the <code>Super</code> constructor.
     *
     * @param title The title of the window. 
     */
    public JPassGenGUI(String title) {

        super(title);
    }


    //////////////////////
    // INSTANCE METHODS //
    //////////////////////

    /** 
     * We center the window. 
     */
    private void center() {

        Dimension screenDim = getToolkit().getScreenSize();
        Dimension frameDim  = getSize();
        Point newLocation = new Point();
        newLocation.x = (screenDim.width - frameDim.width) / 2;
        newLocation.y = (screenDim.height - frameDim.height) / 2;
        setLocation(newLocation);
    }

    /**
     * We update the check boxes display.
     */ 
    private void checkboxesUpdate() {

        for (int ii = 0 ; ii < _arrCheckBoxes.length ; ii++) {
            _arrCheckBoxes[ii].setSelected(_arrChecked[ii]);
        }
    }

    /**
     * We update the check menus display.
     */ 
    private void checkmenusUpdate() {

        for (int ii = 0 ; ii < _arrChecked.length ; ii++) {
            _arrCheckMenus[ii].setSelected(_arrChecked[ii]);
        }
    }

    /** 
     * We enable or not the button and menu "Generate".
     */
    private void enableOrNotGenerate() {

        if (_bSeedOk) {
            boolean enableGenerate = false;
            for (int ii = 0 ; ii < _arrChecked.length ; ii++) {
                if (_arrChecked[ii]) {
                    enableGenerate = true;
                    break;
                }
            }
            _btnGenerate.setEnabled(enableGenerate);
            _menuGenerate.setEnabled(enableGenerate);
        }
    }

    /**
     * The user performs an action on a check box.
     */
    private void doCheckBoxes() {

        for (int ii = 0 ; ii < _arrChecked.length ; ii++) {
            _arrChecked[ii] = _arrCheckBoxes[ii].isSelected();
        }
        this.checkmenusUpdate();
        this.enableOrNotGenerate();
    }

    /**
     * The user performs an action on a check menu.
     */
    private void doCheckMenus() {

        for (int ii = 0 ; ii < _arrChecked.length ; ii++) {
            _arrChecked[ii] = _arrCheckMenus[ii].isSelected();
        }
        this.checkboxesUpdate();
        this.enableOrNotGenerate();
    }

    /** 
     * We send a "about" message box.
     */
    private void doDisplayAbout() {

        String title = LanguageServer.getString("IDS_MENU_ABOUT");
        if (_jopAbout != null) {
            (_jopAbout.createDialog(this, title)).setVisible(true);
        }
    }

    /** 
     * We send a "information" message box.
     */
    private void doDisplayInfos() {

        String title = LanguageServer.getString("IDS_MENU_INFO");
        if (this._jopInfos != null) {
            JDialog jd = _jopInfos.createDialog(this, title);
            jd.setVisible(true);
        }
    }

    /** 
     * We send the "sources" frame.
     */
    private void doDisplaySources() {

        String title = LanguageServer.getString("IDS_MENU_SOURCE");
        JPassGenSource jpgs = new JPassGenSource(title);
        jpgs.setLocationRelativeTo(null);
        jpgs.setVisible(true);
        (new Thread(jpgs)).start();
        //SwingUtilities.invokeLater(new Runnable() { public void run() {
        //}});
    }

    /** 
     * We close the window.
     */
    private void doExit() {

        final int loop = 10;
        Point oldP = getLocation();
        Dimension dim = getSize();

        oldP.x += dim.width  / 2;
        oldP.y += dim.height / 2;

        int dx = dim.width  / loop;
        int dy = dim.height / loop;

        for (int ii = 0 ; ii < loop ; ii++) {
            dim.width  -= dx;
            dim.height -= dy;
            setSize(dim);
            Dimension newDim = getSize();
            Point newP = new Point();
            newP.x = oldP.x - newDim.width  / 2;
            newP.y = oldP.y - newDim.height / 2;
            setLocation(newP);
            try {
                Thread.currentThread().sleep(1000 / loop);
            } catch (InterruptedException e) { ; }
        }
        setVisible(false);
        dispose();
        if (isApplication) {
            System.exit(0);
        }
    }

    /** 
     * We generate the passwords and display them.
     */
    private void doGenerate() {

        // We get what is checked to construct the mask.
        int mask = 0;

        for (int ii = 0 ; ii < _arrChecked.length ; ii++) {
            if (_arrChecked[ii]) {
                mask += _arrMasks[ii];
            }
        }

        // We test the mask.
        if (mask != 0) {
            // We construct the RandomPassword() object.
            RandomPassword rp = new RandomPassword(getCurrentLength(), mask);
            // We fill the text area.
            StringBuffer passwords = new StringBuffer();
            for (int ii = 0 ; ii < Parameters.getCount() ; ii++) {
                passwords.append(rp.getNext());
                if (ii != Parameters.getCount() - 1) { // To avoid scroll
                    passwords.append("\n"); 
                }
            }
            _textPasswords.setText(passwords.toString());
            // We allow passwords  selection.
            _lastLength = rp.getLength();
            _menuSelectOne.setEnabled(true);
            _menuSelectAll.setEnabled(true);
            // We format the entropy.
            DecimalFormat df = new DecimalFormat("#0.00");
            DecimalFormatSymbols dfs = new DecimalFormatSymbols();
            dfs.setDecimalSeparator('.');
            df.setDecimalFormatSymbols(dfs);
            String sEntropy = LanguageServer.getString("IDS_STATUS_ENTROPY") 
                    + " " + df.format(rp.getEntropy()) + " "
                    + LanguageServer.getString("IDS_STATUS_BITS");
            //_textPasswords.setToolTipText(szEntropy);
            _statusBar.setStatus(sEntropy);
            //_textPasswords.selectAll(); // The user will select them himself.
            _textPasswords.repaint();
            _textPasswords.requestFocus();
        } else {
            // Error: nothing has been selected.
            // Now it is useless but I keep it.
            String msg = LanguageServer.getString("IDS_MSGBOX_SELECT");
            (new JOptionPane()).showMessageDialog(this, msg, "Error",
                    JOptionPane.ERROR_MESSAGE);
        }
    }

    /** 
     * We select All passwords.
     */
    private void doSelectAll() {

        _textPasswords.selectAll();
        _textPasswords.requestFocus();
    }

    /** 
     * We select ONE password.
     */
    private void doSelectOne() {

        int index = Math.abs(RandomSingleton.getInstance().nextInt()); 
        index %= Parameters.getCount();
        this.doSelectOne(index);
    }

    /** 
     * We select ONE password.
     * @param index the index of password to select.
     */
    private void doSelectOne(int index) {

        if (0 <= index && index <= Parameters.getCount()) {
            int begin = index * (this.getLastLength() + 1); // 1 is for '\n'.
            int end = begin + this.getLastLength();
            if (0 <= begin && end <= _textPasswords.getText().length()) {
                _textPasswords.select(begin, end);
                _textPasswords.requestFocus();
            }
        }
    }

    /** 
     * We return the current length of passwords to generate.
     *
     * @return the current passwords length.
     */
    private int getCurrentLength() {

        return _sliderLength.getValue();
    }

    /** 
     * We return the last length of generated passwords. 
     *
     * @return the last passwords length.
     */
    private int getLastLength() {

        return _lastLength;
    }

    /** 
     * We create and initialize components and layouts
     * of the main panel.
     */
    private void initializeContentPane() {

        //  We create components.

        // Buttons
        _btnGenerate = new JButton("Generate");
        _btnGenerate.setEnabled(false);
        JButton btnExit = new JButton("Exit");
        //btnExit.setMnemonic(KeyEvent.VK_X);
        this.putMap("IDS_BUTTON_GENERATE", _btnGenerate);
        this.putMap("IDS_BUTTON_EXIT", btnExit);

        // Textarea
        _textPasswords = new JTextArea();
        Font monospaced = new Font("Monospaced", Font.PLAIN,
                _textPasswords.getFont().getSize());
        _textPasswords.setFont(monospaced);
        _textPasswords.setEditable(false);
        _textPasswords.setColumns(Parameters.getMaxLength());
        _textPasswords.setRows(Parameters.getCount());
        _textPasswords.setOpaque(false);

        // We set the background of the main text area.
        ImageIcon imageIcon;
        Color bgcolor;
        ImagePanel imagePanel;
        final int rgb = 220;  //220
        final int alp = 240;  //240
        imageIcon = new ImageIcon(getClass().getResource(JPassGenGUI.ICON));
        bgcolor = new Color(rgb, rgb, rgb, alp);
        imagePanel = new ImagePanel(imageIcon.getImage(), bgcolor);

        int vide = 3;
        Border empty = BorderFactory.createEmptyBorder(vide, vide, vide, vide);
        Border raised = BorderFactory.createRaisedBevelBorder();
        Border lowered = BorderFactory.createLoweredBevelBorder();
        Border compound = BorderFactory.createCompoundBorder(raised, lowered);
        Border border = BorderFactory.createCompoundBorder(compound, empty);

        imagePanel.setBorder(border);
        imagePanel.setLayout(new BorderLayout());
        imagePanel.add(_textPasswords, BorderLayout.CENTER);

        // Check boxes.
        JCheckBox checkboxLowers = new JCheckBox("Lowers");
        JCheckBox checkboxUppers = new JCheckBox("Uppers");
        JCheckBox checkboxDigits = new JCheckBox("Digits");
        JCheckBox checkboxOthers = new JCheckBox("Others");

        this.putMap("IDS_CHECK_LOWERS", checkboxLowers);
        this.putMap("IDS_CHECK_UPPERS", checkboxUppers);
        this.putMap("IDS_CHECK_DIGITS", checkboxDigits);
        this.putMap("IDS_CHECK_OTHERS", checkboxOthers);

        // We create an array of check boxes
        _arrCheckBoxes = new JCheckBox[] {checkboxLowers, checkboxUppers,
                checkboxDigits , checkboxOthers};

        // Slider.
        _sliderLength = new JSlider(java.awt.Adjustable.HORIZONTAL);
        _sliderLength.setMinimum(Parameters.getMinLength());
        _sliderLength.setMaximum(Parameters.getMaxLength());
        _sliderLength.setValue(_sliderLength.getMinimum());
        _sliderLength.setMajorTickSpacing(4); 
        _sliderLength.setMinorTickSpacing(1);
        _sliderLength.setPaintTicks(true);
        _sliderLength.setPaintLabels(true);
        _sliderLength.setValueIsAdjusting(true);
        _sliderLength.setSnapToTicks(true);

        this.putMap("IDS_SLIDER_TIPS", _sliderLength);

        // We add listeners...

        // For the window.
        addWindowListener(
            new WindowAdapter() {
                public void windowActivated(WindowEvent we) { }
                public void windowClosing(WindowEvent we) {
                    doExit();
                }
            }
        );
        // For the check boxes.
        ActionListener alCheckBoxes = new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                doCheckBoxes();
            }
        };
        for (int ii = 0 ; ii < _arrCheckBoxes.length ; ii++) { 
            _arrCheckBoxes[ii].addActionListener(alCheckBoxes);
        }
        // For the "Generate" button.
        _btnGenerate.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                doGenerate();
            }
        });

        // For the button "Exit".
        btnExit.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                doExit();
            }
        });

        // For a mouse click on the main text area.
        MouseListener mltextPassword = new MouseAdapter() {
            public void mouseClicked(MouseEvent me) {
                // We select one password.
                int index = me.getY() * Parameters.getCount();
                index /= _textPasswords.getSize().height;
                doSelectOne(index);
            }
        };
        _textPasswords.addMouseListener(mltextPassword);

        // We create the status bar.
        _statusBar = new StatusBar();

        // We set the layouts.

        // East panel.
        JPanel panelEast = new JPanel();
        panelEast.setLayout(new GridLayout(6, 1));

        panelEast.add(checkboxLowers);
        panelEast.add(checkboxUppers);
        panelEast.add(checkboxDigits);
        panelEast.add(checkboxOthers);
        panelEast.add(_btnGenerate);
        panelEast.add(btnExit);

        // Main panel without status bar.
        JPanel withoutStatusBar = new JPanel();
        withoutStatusBar.setLayout(new BorderLayout());

        withoutStatusBar.add(panelEast, BorderLayout.EAST);
        withoutStatusBar.add(imagePanel, BorderLayout.CENTER);
        withoutStatusBar.add(_sliderLength, BorderLayout.SOUTH);

        // Main panel (with status bar).
        JPanel mainPanel = (JPanel) getContentPane();
        mainPanel.setLayout(new BorderLayout());
        mainPanel.add(withoutStatusBar, BorderLayout.CENTER);
        mainPanel.add(_statusBar, BorderLayout.SOUTH);
    }

    /** 
     * We create and initialize the menu of the frame 
     */
    private void initializeMenu() {

        JMenuBar  menuBar;
        JMenu     menu;
        JMenuItem menuInfos;
        JMenuItem menuExit;
        JMenuItem menuSource;
        JMenuItem menuAbout;

        _idsMap = new HashMap();
        // We create a new menu bar.
        menuBar = new JMenuBar();

        // We add the "File" menu.
        menu = new JMenu("File");
        _menuGenerate = new JMenuItem("Generate");
        menuExit = new JMenuItem("Exit");
        menu.add(_menuGenerate);
        menu.addSeparator();
        menu.add(menuExit);
        menuBar.add(menu);
        _menuGenerate.setEnabled(false);

        this.putMap("IDS_MENU_GENERATE", _menuGenerate);
        this.putMap("IDS_MENU_EXIT", menuExit);
        this.putMap("IDS_MENU_FILE", menu);

        // We add the "Edit" Menu.
        menu = new JMenu("Edit");
        _menuSelectOne = new JMenuItem("Select One");
        _menuSelectAll = new JMenuItem("Select All");
        menu.add(_menuSelectOne);
        menu.add(_menuSelectAll);
        menuBar.add(menu);
        _menuSelectOne.setEnabled(false);
        _menuSelectAll.setEnabled(false);

        this.putMap("IDS_MENU_EDIT", menu);
        this.putMap("IDS_MENU_SELECT_ALL", _menuSelectAll);
        this.putMap("IDS_MENU_SELECT_ONE", _menuSelectOne);

        // We add the "Options" Menu.
        menu = new JMenu("Options");
        JCheckBoxMenuItem checkmenuLowers = new JCheckBoxMenuItem("Lowers");
        JCheckBoxMenuItem checkmenuUppers = new JCheckBoxMenuItem("Uppers");
        JCheckBoxMenuItem checkmenuDigits = new JCheckBoxMenuItem("Digits");
        JCheckBoxMenuItem checkmenuOthers = new JCheckBoxMenuItem("Others");
        menu.add(checkmenuLowers);
        menu.add(checkmenuUppers);
        menu.add(checkmenuDigits);
        menu.add(checkmenuOthers);
        menuBar.add(menu);

        this.putMap("IDS_MENU_OPTIONS", menu);
        this.putMap("IDS_MENU_LOWERS", checkmenuLowers);
        this.putMap("IDS_MENU_UPPERS", checkmenuUppers);
        this.putMap("IDS_MENU_DIGITS", checkmenuDigits);
        this.putMap("IDS_MENU_OTHERS", checkmenuOthers);

        // We create a array of check menu item.
        _arrCheckMenus = new JCheckBoxMenuItem[] {checkmenuLowers,
                checkmenuUppers, checkmenuDigits, checkmenuOthers};

        // We add the "Help" menu.
        menu = new JMenu("Help");
        menuInfos = new JMenuItem("Infos...");
        menuSource = new JMenuItem("Sources...");
        menuAbout = new JMenuItem("About...");
        menu.add(menuInfos);
        menu.add(menuSource);
        menu.add(menuAbout);
        menuBar.add(menu);

        this.putMap("IDS_MENU_HELP", menu);
        this.putMap("IDS_MENU_INFO", menuInfos);
        this.putMap("IDS_MENU_SOURCE", menuSource);
        this.putMap("IDS_MENU_ABOUT", menuAbout);

        // We add listeners...

        // We add listeners for the "Generate" menu item.
        _menuGenerate.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                doGenerate();
            }
        });

        // We add listeners for the "Exit" menu item.
        menuExit.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                doExit();
            }
        });

        // We add listeners for the "Select one" menu item.
        _menuSelectOne.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                doSelectOne();
            }
        });

        // We add listeners for the "Select All" menu item.
        _menuSelectAll.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                doSelectAll();
            }
        });

        // We add a listener for the check menus items.
        ActionListener alCheckMenus = new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                doCheckMenus();
            }
        };
        for (int ii = 0 ; ii < _arrCheckMenus.length ; ii++) {
            _arrCheckMenus[ii].addActionListener(alCheckMenus);
        }

        // We add a listener for the "Infos" item menu.
        menuInfos.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                doDisplayInfos();
            }
        });

        // We add a listener for the "Sources" item menu.
        menuSource.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                doDisplaySources();
            }
        });

        // We add a listener for the "About" item menu.
        menuAbout.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                doDisplayAbout();
            }
        });

        // We add menu to the main frame.
        setJMenuBar(menuBar);
    }

    /**
     * We initialize the content of the message boxes.
     */
    private void initializeMessageBox() {

        StringBuffer buf = new StringBuffer();

        // "Infos" Option Pane.
        String othersList =  RandomPassword.OTHERS_LIST;
        othersList = othersList.replaceAll("<", "&lt;");
        othersList = othersList.replaceAll(">", "&gt");
        othersList = othersList.replaceAll(" ", "&nbsp;");
        buf.append("<html>");
        buf.append("<tt>");
        buf.append("<table border='1'>");
        buf.append("<tr>");
        buf.append("<td>"); 
        buf.append(LanguageServer.getString("IDS_CHECK_LOWERS"));
        buf.append("</td>");
        buf.append("<td>" + RandomPassword.LOWERS_LIST + "</td>");
        buf.append("</tr>");
        buf.append("<tr>");
        buf.append("<td>"); 
        buf.append(LanguageServer.getString("IDS_CHECK_UPPERS"));
        buf.append("</td>");
        buf.append("<td>" + RandomPassword.UPPERS_LIST + "</td>");
        buf.append("</tr>");
        buf.append("<tr>");
        buf.append("<td>"); 
        buf.append(LanguageServer.getString("IDS_CHECK_DIGITS"));
        buf.append("</td>");
        buf.append("<td>" + RandomPassword.DIGITS_LIST + "</td>");
        buf.append("</tr>");
        buf.append("<tr>");
        buf.append("<td>"); 
        buf.append(LanguageServer.getString("IDS_CHECK_OTHERS"));
        buf.append("</td>");
        buf.append("<td>" + othersList + "</td>");
        buf.append("</tr>");
        buf.append("<tr>");
        buf.append("<td>"); 
        buf.append(LanguageServer.getString("IDS_INFO_ALGORITHM"));
        buf.append("</td>");
        buf.append("<td>" + RandomSingleton.ALGO_RANDOM + "</td>");
        buf.append("</tr>");
        buf.append("</table>");
        buf.append("</tt>");
        buf.append("</html>");

        othersList = null;

        _jopInfos = new JOptionPane(buf.toString().intern(), 
                JOptionPane.PLAIN_MESSAGE, JOptionPane.DEFAULT_OPTION);

        buf.delete(0, buf.length());

        // We format the version.
        DecimalFormat df = new DecimalFormat("#0.00");
        DecimalFormatSymbols dfs = new DecimalFormatSymbols();
        dfs.setDecimalSeparator('.');
        df.setDecimalFormatSymbols(dfs);

        // "About" Option Pane.
        buf.append("<html>");
        buf.append("*&nbsp;");
        buf.append("<font color='#0000CD'>");
        buf.append("JPassGen");
        buf.append("</font>");
        buf.append("&nbsp;");
        buf.append("<font color='#FF6347'>");
        buf.append(df.format(JPassGenGUI.VERSION));
        buf.append("</font>");
        buf.append("&nbsp;");
        buf.append("<font color='#D02090'>");
        buf.append("- GPL 2002");
        buf.append("</font>");
        buf.append("<br>");
        buf.append("<font color='#8420FF'>");
        buf.append("http://jpassgen.sourceforge.net/");
        buf.append("</font>");
        buf.append("<br>");
        buf.append("*&nbsp;");
        buf.append("<font color='#800080'>");
        buf.append("Kunststoff Look&Feel v2.0");
        buf.append("</font>");
        buf.append("<br>");
        buf.append("<font color='#BA55D3'>");
        buf.append("http://www.incors.org/");
        buf.append("</font>");
        buf.append("<br>");
        buf.append("*&nbsp;");
        buf.append("<font color='#006600'>");
        buf.append("Icons Copyright(C) 1998 by Dean S.Jones");
        buf.append("</font>");
        buf.append("<br>");
        buf.append("<font color='#663333'>");
        buf.append("http://sourceforge.net/projects/icon-collection/");
        buf.append("</font>");
        buf.append("</html>");

        URL urlIcon = getClass().getResource(JPassGenGUI.ICON);
        _jopAbout = new JOptionPane(buf.toString().intern(), 
                JOptionPane.INFORMATION_MESSAGE, JOptionPane.DEFAULT_OPTION,
                new ImageIcon(urlIcon));

        buf.delete(0, buf.length());
        buf = null;
    }

    /**
     * We initialize the seed in a new thread (lazy)
     * and update the gui.
     */
    void initializeSeed() {

        final String msg1 = LanguageServer.getString("IDS_STATUS_WAITING");
        final String msg2 = LanguageServer.getString("IDS_STATUS_OK");
        Runnable runner = new Runnable() {
            public void run() {
                SwingUtilities.invokeLater(new Runnable() {
                    public void run() {
                        _statusBar.setStatus(msg1);
                    }
                });
                RandomSingleton rg = RandomSingleton.getInstance();
                _bSeedOk = true;
                SwingUtilities.invokeLater(new Runnable() {
                    public void run() {
                        _statusBar.setStatus(msg2);
                        enableOrNotGenerate();
                    }
                });
            }
        };
        (new Thread(runner)).start();
    }

    /**
     * We initialize the user interface. 
     */
    void initializeUi() {

        setResizable(false);

        this.initializeMessageBox();
        this.initializeMenu();
        this.initializeContentPane();
        //this.initializeStrings();

        URL urlIcon = getClass().getResource(JPassGenGUI.ICON);
        setIconImage(new ImageIcon(urlIcon).getImage());

        pack();

        this.center();
        this.setInitialUi();
        this.setWelcome();
    }

    /**
     * We add a component to the map with its string identifier.
     * @param ids a string identifier
     * @param comp a component
     */
    void putMap(String ids, JComponent comp) {

        _idsMap.put(ids, comp);
        if (comp instanceof AbstractButton) {
            ((AbstractButton) comp).setText(LanguageServer.getString(ids));
        } else if (comp instanceof JSlider) {
            String tips = LanguageServer.getString(ids);
            ((JSlider) comp).setToolTipText(tips);
        }
    }

    /** 
     * We set the current length of passwords to generate.
     *
     * @param length The new length of the passwords.
     */
    private void setCurrentLength(int length) {

        if (Parameters.getMinLength() <= length 
                  && length <= Parameters.getMaxLength()) {
            _sliderLength.setValue(length);
        }
    }

    /** 
     * We modify the window options to default.
     */
    private void setInitialUi() {

        // "Strong passwords"
        //this.setOptions(RandomPassword.ALLS, Parameters.getMaxLength());
        this.setOptions(Parameters.getInitialMask(), 
                Parameters.getInitialLength());
    }

    /** 
     * We modify the window options.
     * @param mask the password mask 
     * @param length the password length
     */
    private void setOptions(int mask, int length) {

        // We check characters.
        for (int ii = 0 ; ii < RandomPassword.MASK_ARRAY.length ; ii++) {
            int test = RandomPassword.MASK_ARRAY[ii];
            this._arrChecked[ii] = ((mask & test) == test);
        }

        this.enableOrNotGenerate();

        // We update check components.
        this.checkboxesUpdate();
        this.checkmenusUpdate();

        // We set length to maximum.
        this.setCurrentLength(length);
    }

    /** 
     * We display welcome informations in the main text area.
     */
    void setWelcome() {

        StringBuffer buf = new StringBuffer();
        final int lastLine = 10;
        for (int ii = 0 ; ii < lastLine ; ii++) {
            String ids = "IDS_WELCOME_" + ii;
            buf.append(LanguageServer.getString(ids));
            if (ii != lastLine - 1) {
                buf.append("\n");
            }
        }
        _textPasswords.setText(buf.toString());
    }

    /**
     * We update the strings for visual components.
     */
    void updateStrings() {

        Set set = _idsMap.keySet();
        for (Iterator iterator = set.iterator() ; iterator.hasNext() ;) {
            Object key = iterator.next(); 
            Object obj = _idsMap.get(key);
            String strKey = (String) key;
            String text = LanguageServer.getString(strKey);
            if (obj instanceof AbstractButton) {
                ((AbstractButton) obj).setText(text);
            } else if (obj instanceof JSlider) {
                ((JSlider) obj).setToolTipText(text);
            }
        }
        setTitle(LanguageServer.getString("IDS_TITLE_APP"));
        pack();
    }

} // End of JPassGenGUI class

//////////////
// CODE RIP //
//////////////

/* 
// transparency test
//_textPasswords.setBackground(new java.awt.Color(255, 255, 255, 220));
//_textPasswords.setOpaque(true);

// Scroll for Textarea (I keep it for later)
//final ImageIcon imageIcon = new ImageIcon(getClass().
// getResource(JPassGenGUI.ICON));
//final int winc = imageIcon.getIconWidth();
//final int hinc = imageIcon.getIconHeight();

JScrollPane scrollTextPasswords = new JScrollPane() {

    public void update(java.awt.Graphics g) {
      int w = this.getParent().getWidth();
      int h = this.getParent().getHeight();
      for (int i = 0; i < h + hinc; i = i + hinc)
        for (int j = 0; j < w + winc; j = j + winc)
          imageIcon.paintIcon(this, g, i, j);
      _textPasswords.invalidate();
      _textPasswords.validate();
      _textPasswords.repaint();
    }
    public void paint(java.awt.Graphics g) {
        update(g);
    }
    public void paintComponents(java.awt.Graphics g) {
        update(g);
    }
};

javax.swing.UIDefaults defaults = UIManager.getDefaults();
for (java.util.Enumeration e = defaults.keys(); e.hasMoreElements() ;) {
    Object objKey =  e.nextElement();
    //System.out.println(defaults.get(key).toString());
    if (objKey instanceof String && ((String) objKey).indexOf("font") != -1 ) {
        String strKey = (String) objKey;
        // System.out.println(strKey + "=" + defaults.get(strKey).toString());
    }
    //System.out.println(key);
}

// JScrollPane scrollTextPasswords = new JScrollPane();
JScrollPane scrollTextPasswords = new JScrollPane();
scrollTextPasswords.setVerticalScrollBarPolicy(
        JScrollPane.VERTICAL_SCROLLBAR_NEVER);
scrollTextPasswords.setHorizontalScrollBarPolicy(
        JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
scrollTextPasswords.setViewportView(imagePanel);
//scrollTextPasswords.setBackground(java.awt.Color.BLACK);

//Font monospaced = new Font("Monospaced", Font.PLAIN,
//        _textPasswords.getFont().getSize());
//_textPasswords.setForeground(Color.BLACK);
//_textPasswords.setDoubleBuffered(true);
//_textPasswords.setFont(monospaced);

*/

// End of JPassGenGUI.java file.
