/*
 * @(#)JPassGenSource.java  2002-12-19
 *
 * Version 1.04
 *
 * Copyright (C) 26 juillet 2002 hughes monget
 * hmonget@users.sourceforge.net
 * http://jpassgen.sourceforge.net/
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package net.sourceforge.jpassgen;

// For GUI components.
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Panel;
import java.awt.BorderLayout;

// For GUI components.
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
//import javax.swing.JTextArea;
import javax.swing.JProgressBar;
import javax.swing.JEditorPane;
import javax.swing.SwingUtilities;

// For the icon
import java.net.URL;

// For the list of files.
import java.util.Properties;

// To read source codes.
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.IOException;

// To colorize code sources.
import com.coolservlets.CodeViewer;

/**
 * This class is the frame for displaying the source codes of the application.
 *
 * @author hughes monget
 */
class JPassGenSource extends JFrame implements Runnable {


    //////////////////
    // CLASS FIELDS //
    //////////////////

    /**
     * The width of the frame.
     */
    public static final int WIDTH = 640;

    /**
     * The height of the frame.
     */
    public static final int HEIGHT = WIDTH / 4 * 3;


    ///////////////////
    // CLASS METHODS //
    ///////////////////

    /** 
     * Main function . <br>
     *
     * @param args command line arguments. 
     */ 
    public static void main(String args[]) {

        // Used for tests.
        JPassGenGUI.setLookAndFeel();
        JPassGenSource jpgs = new JPassGenSource("test");
        jpgs.setLocationRelativeTo(null);
        jpgs.setVisible(true);
        (new Thread(jpgs)).start();
    }


    /////////////////////
    // INSTANCE FIELDS //
    /////////////////////

    /**
     * The main tabbed pane of the frame.
     */
    private JTabbedPane _tabbedPane;

    /**
     * The progress bar.
     */
    private JProgressBar _progressBar;

    //////////////////
    // CONSTRUCTORS //
    //////////////////

    /** 
     * Constructor.
     * @param title the title of the window.
     */ 
    JPassGenSource(String title) {

        super(title);
        setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        URL urlIcon = getClass().getResource(JPassGenGUI.ICON);
        setIconImage(new ImageIcon(urlIcon).getImage());

        _tabbedPane = new JTabbedPane(JTabbedPane.LEFT);
        _progressBar = new JProgressBar();
        _progressBar.setValue(0);
        _progressBar.setMinimum(0);
        Panel panel = new Panel();
        panel.setLayout(new BorderLayout());
        panel.add(_tabbedPane, BorderLayout.CENTER);
        panel.add(_progressBar, BorderLayout.SOUTH);
        getContentPane().add(panel);
        setSize(new Dimension(JPassGenSource.WIDTH, JPassGenSource.HEIGHT));
    }

    //////////////////////
    // INSTANCE METHODS //
    //////////////////////

    /**
     * We initialize the frame.
     */
    public void run() {

        Properties listFiles = new Properties();
        String jpsrc = "sources.properties";
        InputStream filePr = getClass().getResourceAsStream(jpsrc);
        // We get the list of files.
        if (filePr != null) {
            try {
                listFiles.load(filePr);
            } catch (java.io.IOException e) { 
                System.err.println("** IOException" + e.getMessage()); 
            }
            String count = listFiles.getProperty("JPassGen.file.count");
            int filesCount;
            if (count != null) {
                filesCount = Integer.valueOf(count).intValue();
                _progressBar.setMaximum(filesCount);
                final Font font = new Font("Monospaced", Font.PLAIN, 12);
                for (int ii = 0 ; ii < filesCount ; ii++) {
                    CodeViewer cv = new CodeViewer();
                    String key = "JPassGen.file." + ii;
                    final String filename = listFiles.getProperty(key);
                    final JScrollPane jsp = new JScrollPane();
                    JEditorPane jta = new JEditorPane("text/html", "");
                    //JTextArea jta = new JTextArea();
                    //jta.setFont(font);
                    jta.setEditable(false);
                    jsp.setViewportView(jta);

                    try {
                        String path = "src/" + filename;
                        InputStream is = getClass().getResourceAsStream(path);
                        if (is != null) {

                            StringBuffer tmp = new StringBuffer(); 
                            InputStreamReader isr = new InputStreamReader(is); 
                            BufferedReader br = new BufferedReader(isr);
                            tmp.append("<html><head><title>");
                            tmp.append(filename);
                            tmp.append("</title></head>");
                            tmp.append("<body bgcolor=\"#CCCCCC\"><pre>");
                            String line = br.readLine();
                            while (line != null) {
                                if (filename.endsWith(".java")) {
                                    line = cv.syntaxHighlight(line);
                                }
                                tmp.append(line);
                                tmp.append("\n");
                                line = br.readLine();
                            }
                            tmp.append("</pre></body></html>");
                            jta.setText(tmp.toString());
                            //jta.read(br, null);
                            br.close();
                            isr.close();

                            final int value = ii + 1;
                            Runnable runner =  new Runnable() {  
                                public void run() {
                                    invalidate();
                                    _tabbedPane.add(filename, jsp);
                                    _progressBar.setValue(value);
                                    validate();
                                    repaint();
                                }
                            };
                            SwingUtilities.invokeLater(runner);
                        }
                    } catch (IOException e) {
                        System.err.println("** IOException:" + e.getMessage());
                    }
                }
            }
        } else {
            System.err.println("** listFiles not found !");
        }
    }

}

// End of JPassGenSource.java file.
