/*
 * @(#)LanguageServer.java  2002-12-24
 *
 * Version 1.04
 *
 * Copyright (C) 26 juillet 2002 hughes monget
 * hmonget@users.sourceforge.net
 * http://jpassgen.sourceforge.net/
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package net.sourceforge.jpassgen;

import java.util.MissingResourceException;
import java.util.ResourceBundle;

/**
 * This class is a "language server" for all the application.
 *
 * @author hughes monget
 */
class LanguageServer {

    //////////////////
    // CLASS FIELDS //
    //////////////////

    /**
     * The base name of the file containing the string identifiers (IDS)
     */
    private static final String IDS_FILENAME = "strings";

    /**
     * The singleton instance.
     */
    private static LanguageServer singleton = null;


    ///////////////////
    // CLASS METHODS //
    ///////////////////

    /**
     * The "getInstance()" method of "Singleton pattern"
     * @return the strings server singleton. 
     */
    static synchronized LanguageServer getInstance() {

        if (LanguageServer.singleton == null) {
            LanguageServer.singleton = new LanguageServer();
        }
        return LanguageServer.singleton;
    }

    /**
     * We return a string (or "** IDS" if not found.)
     *
     * @param ids the string identifier (IDS)
     * @return the string found or not.
     */
    static String getString(String ids) {

        return LanguageServer.getInstance().getString(
                LanguageServer.IDS_FILENAME, ids);
    }

    //////////////////
    // CONSTRUCTORS //
    //////////////////

    /** 
     * The default constructor is private.
     */
    private LanguageServer() { ; }

    //////////////////////
    // INSTANCE METHODS //
    //////////////////////

    /**
     * We return a string (or "** IDS" if not found.)
     * from a properties file localized in the main package.
     *
     * @param basename the base name of the properties file.
     * @param ids the string identifier (IDS)
     * @return the string found or not.
     */
    String getString(String basename, String ids) {

        String ret = "** " + ids;
        String fullpath = getClass().getPackage().getName() + "." + basename;
        try {
            ResourceBundle rb = ResourceBundle.getBundle(fullpath);
            ret = rb.getString(ids);
            //ret = ids;
        } catch (MissingResourceException e) {
             System.err.println("** MissingResourceException:" 
                     + e.getMessage());
        }
        return ret;
    }

} // End of LanguageServer.java

