/*
 * @(#)SplashWindow.java  2002-07-29
 *
 * Version 1.04
 *
 * Copyright (C) 26 juillet 2002 hughes monget
 * hmonget@users.sourceforge.net
 * http://jpassgen.sourceforge.net/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package net.sourceforge.jpassgen;

import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Graphics;
import java.awt.Toolkit;
import java.awt.Font;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JWindow;
import javax.swing.SwingConstants;

/**
 * This class is a simple splash screen. I don't care thread problems...
 *
 * @author hughes monget
 */
class SplashWindow extends JWindow {

    ///////////////////
    // CLASS METHODS //
    ///////////////////

    /**
     * To test the splash screen
     * @param args arguments command line
     */
    public static void main(String args[]) {

        SplashWindow dirtyTest = new SplashWindow("foo");
        dirtyTest.setText("bar");
        dirtyTest.setVisible(true);
    }


    /////////////////////
    // INSTANCE FIELDS //
    /////////////////////

    /**
     * This label displays the message send to the user.
     */
    private JLabel _label;

    /**
     * The width of the splash.
     */ 
    private int _width = 160;

    /**
     * The height of the splash.
     */ 
    private int _height = 100;


    //////////////////
    // CONSTRUCTORS //
    //////////////////

    /**
     * Default constructor: we initialize the window.
     * @param title title application.
     */
    SplashWindow(String title) {

        setSize(_width, _height);

        Container container = new Container() {

            public void paint(Graphics g) {

                super.paint(g);

                Dimension dim = this.getSize();

                Color beginColor   = Color.GRAY;
                Color endColor     = Color.LIGHT_GRAY;

                float step = dim.height;

                float deltaR = (endColor.getRed()   - beginColor.getRed());
                float deltaG = (endColor.getGreen() - beginColor.getGreen());
                float deltaB = (endColor.getBlue()  - beginColor.getBlue());

                float beginR = beginColor.getRed()   / 256.0f;
                float beginG = beginColor.getGreen() / 256.0f;
                float beginB = beginColor.getBlue()  / 256.0f;

                float valueR;
                float valueG;
                float valueB;

                deltaR /= (step * 256.0f);
                deltaG /= (step * 256.0f);
                deltaB /= (step * 256.0f);

                for (int ii = 0 ; ii < dim.height ; ii++) {

                    valueR = beginR + deltaR * ii;
                    valueG = beginG + deltaG * ii;
                    valueB = beginB + deltaB * ii;

                    valueR = (valueR > 1.0f) ? 1.0f : valueR;
                    valueG = (valueG > 1.0f) ? 1.0f : valueG;
                    valueB = (valueB > 1.0f) ? 1.0f : valueB;

                    valueR = (valueR < 0.0f) ? 0.0f : valueR;
                    valueG = (valueG < 0.0f) ? 0.0f : valueG;
                    valueB = (valueB < 0.0f) ? 0.0f : valueB;

                    g.setColor(new Color(valueR, valueG, valueB));

                    g.drawLine(0, ii, dim.width, ii);
                }
                super.paint(g);
            }
        };

        URL urlIcon = getClass().getResource(JPassGenGUI.ICON);
        ImageIcon imageIcon = new ImageIcon(urlIcon);

        JLabel jlTitle = new JLabel(title, imageIcon, SwingConstants.CENTER);

        _label = new JLabel(" ", SwingConstants.CENTER);

        jlTitle.setFont(jlTitle.getFont().deriveFont(Font.BOLD));

        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints gbc = new GridBagConstraints();

        container.setLayout(gridbag);

        gbc.gridx = GridBagConstraints.REMAINDER;
        gbc.weighty = 3;

        gbc.fill = GridBagConstraints.BOTH;

        gridbag.setConstraints(jlTitle, gbc);
        container.add(jlTitle);

        gbc.weighty = 1;

        gridbag.setConstraints(_label, gbc);
        container.add(_label);

        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

        setLocation((screenSize.width - getSize().width) / 2,
                    (screenSize.height - getSize().height) / 2);

        addMouseListener(
            new MouseAdapter() {
                public void mousePressed(MouseEvent e) {
                    setVisible(false);
                }
            }
        );

        setContentPane(container);
    }


    //////////////////////
    // INSTANCE METHODS //
    //////////////////////

    /**
     * We set the message to display to the user.
     *
     * @param message the message to send.
     */
    void setText(String message) {

        _label.setText(message);
    }

} // End of SplashWindow.java
